package xyz.lychee.lagfixer.libs.adventure.translation;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Stream;
import xyz.lychee.lagfixer.libs.adventure.internal.Internals;
import xyz.lychee.lagfixer.libs.adventure.key.Key;
import xyz.lychee.lagfixer.libs.adventure.util.TriState;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.examination.Examinable;
import xyz.lychee.lagfixer.libs.examination.ExaminableProperty;

final class TranslationRegistryImpl implements Examinable, TranslationRegistry {
   private final Key name;
   private final Map<String, Translation> translations = new ConcurrentHashMap();
   private Locale defaultLocale;

   TranslationRegistryImpl(final Key name) {
      this.defaultLocale = Locale.US;
      this.name = name;
   }

   public void register(final @NotNull String key, final @NotNull Locale locale, final @NotNull MessageFormat format) {
      ((Translation)this.translations.computeIfAbsent(key, (x$0) -> {
         return new Translation(x$0);
      })).register(locale, format);
   }

   public void unregister(final @NotNull String key) {
      this.translations.remove(key);
   }

   public @NotNull Key name() {
      return this.name;
   }

   public boolean contains(final @NotNull String key) {
      return this.translations.containsKey(key);
   }

   public @NotNull TriState hasAnyTranslations() {
      return !this.translations.isEmpty() ? TriState.TRUE : TriState.FALSE;
   }

   public @Nullable MessageFormat translate(final @NotNull String key, final @NotNull Locale locale) {
      Translation translation = (Translation)this.translations.get(key);
      return translation == null ? null : translation.translate(locale);
   }

   public void defaultLocale(final @NotNull Locale defaultLocale) {
      this.defaultLocale = (Locale)Objects.requireNonNull(defaultLocale, "defaultLocale");
   }

   public @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
      return Stream.of(ExaminableProperty.of("translations", this.translations));
   }

   public boolean equals(final Object other) {
      if (this == other) {
         return true;
      } else if (!(other instanceof TranslationRegistryImpl)) {
         return false;
      } else {
         TranslationRegistryImpl that = (TranslationRegistryImpl)other;
         return this.name.equals(that.name) && this.translations.equals(that.translations) && this.defaultLocale.equals(that.defaultLocale);
      }
   }

   public int hashCode() {
      return Objects.hash(new Object[]{this.name, this.translations, this.defaultLocale});
   }

   public String toString() {
      return Internals.toString(this);
   }

   final class Translation implements Examinable {
      private final String key;
      private final Map<Locale, MessageFormat> formats;

      Translation(final String key) {
         this.key = (String)Objects.requireNonNull(key, "translation key");
         this.formats = new ConcurrentHashMap();
      }

      void register(final @NotNull Locale locale, final @NotNull MessageFormat format) {
         if (this.formats.putIfAbsent((Locale)Objects.requireNonNull(locale, "locale"), (MessageFormat)Objects.requireNonNull(format, "message format")) != null) {
            throw new IllegalArgumentException(String.format("Translation already exists: %s for %s", this.key, locale));
         }
      }

      @Nullable MessageFormat translate(final @NotNull Locale locale) {
         MessageFormat format = (MessageFormat)this.formats.get(Objects.requireNonNull(locale, "locale"));
         if (format == null) {
            format = (MessageFormat)this.formats.get(new Locale(locale.getLanguage()));
            if (format == null) {
               format = (MessageFormat)this.formats.get(TranslationRegistryImpl.this.defaultLocale);
               if (format == null) {
                  format = (MessageFormat)this.formats.get(TranslationLocales.global());
               }
            }
         }

         return format;
      }

      public @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
         return Stream.of(ExaminableProperty.of("key", this.key), ExaminableProperty.of("formats", this.formats));
      }

      public boolean equals(final Object other) {
         if (this == other) {
            return true;
         } else if (!(other instanceof Translation)) {
            return false;
         } else {
            Translation that = (Translation)other;
            return this.key.equals(that.key) && this.formats.equals(that.formats);
         }
      }

      public int hashCode() {
         return Objects.hash(new Object[]{this.key, this.formats});
      }

      public String toString() {
         return Internals.toString(this);
      }
   }
}
